local _G = getfenv(0)

local YurrCombatLog = LibStub("AceAddon-3.0"):NewAddon("YurrCombatLog", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0")
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("YurrCombatLog", true)

local debugf = tekDebug and tekDebug:GetFrame("YurrCombatLog")
local function Debug(...) if debugf then debugf:AddMessage(string.join(", ", ...)) end end

local CombatLogQuickButtonFrameProgressBar
--local CombatLogUpdateFrame = CreateFrame("Frame", "CombatLogUpdateFrame", UIParent)

local prefixList
local suffixList
local spellList
local schools = {
	[SCHOOL_MASK_PHYSICAL]						= STRING_SCHOOL_PHYSICAL,
	[SCHOOL_MASK_HOLY]							= STRING_SCHOOL_HOLY,
	[SCHOOL_MASK_FIRE]							= STRING_SCHOOL_FIRE,
	[SCHOOL_MASK_NATURE]						= STRING_SCHOOL_NATURE,
	[SCHOOL_MASK_FROST]							= STRING_SCHOOL_FROST,
	[SCHOOL_MASK_SHADOW]						= STRING_SCHOOL_SHADOW,
	[SCHOOL_MASK_ARCANE]						= STRING_SCHOOL_ARCANE,
	-- Physical and a Magical
	[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FIRE]	= STRING_SCHOOL_FLAMESTRIKE,
	[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FROST]	= STRING_SCHOOL_FROSTSTRIKE,
	[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_ARCANE]	= STRING_SCHOOL_SPELLSTRIKE,
	[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_NATURE]	= STRING_SCHOOL_STORMSTRIKE,
	[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_SHADOW]	= STRING_SCHOOL_SHADOWSTRIKE,
	[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_HOLY]	= STRING_SCHOOL_HOLYSTRIKE,
	-- Two Magical Schools
	[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST]		= STRING_SCHOOL_FROSTFIRE,
	[SCHOOL_MASK_FIRE + SCHOOL_MASK_ARCANE]		= STRING_SCHOOL_SPELLFIRE,
	[SCHOOL_MASK_FIRE + SCHOOL_MASK_NATURE]		= STRING_SCHOOL_FIRESTORM,
	[SCHOOL_MASK_FIRE + SCHOOL_MASK_SHADOW]		= STRING_SCHOOL_SHADOWFLAME,
	[SCHOOL_MASK_FIRE + SCHOOL_MASK_HOLY]		= STRING_SCHOOL_HOLYFIRE,
	[SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE]	= STRING_SCHOOL_SPELLFROST,
	[SCHOOL_MASK_FROST + SCHOOL_MASK_NATURE]	= STRING_SCHOOL_FROSTSTORM,
	[SCHOOL_MASK_FROST + SCHOOL_MASK_SHADOW]	= STRING_SCHOOL_SHADOWFROST,
	[SCHOOL_MASK_FROST + SCHOOL_MASK_HOLY]		= STRING_SCHOOL_HOLYFROST,
	[SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE]	= STRING_SCHOOL_SPELLSTORM,
	[SCHOOL_MASK_ARCANE + SCHOOL_MASK_SHADOW]	= STRING_SCHOOL_SPELLSHADOW,
	[SCHOOL_MASK_ARCANE + SCHOOL_MASK_HOLY]		= STRING_SCHOOL_DIVINE,
	[SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW]	= STRING_SCHOOL_SHADOWSTORM,
	[SCHOOL_MASK_NATURE + SCHOOL_MASK_HOLY]		= STRING_SCHOOL_HOLYSTORM,
	[SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY]		= STRING_SCHOOL_SHADOWLIGHT,
	-- Three or more schools
	[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_NATURE]																						= STRING_SCHOOL_ELEMENTAL,
	[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW]											= STRING_SCHOOL_CHROMATIC,
	[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY]						= STRING_SCHOOL_MAGIC,
	[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY]	= STRING_SCHOOL_CHAOS
}
local powerTypes = {
	[-2] = string.lower(HEALTH),
	[0] = string.lower(MANA),
	[1] = string.lower(RAGE),
	[2] = string.lower(FOCUS),
	[3] = string.lower(ENERGY),
	[4] = string.lower(format("%s%s", PET, HAPPINESS)),
	[5] = string.lower(RUNES),
	[6] = string.lower(RUNIC_POWER),
}
local classColors = {}

local defaults = {
	profile = {
		version = 0,
		outputFrame = "ChatFrame2",
		unitColors = {
			[1297] = "8080FF", -- self
			[4991] = "804D26", -- pet
			[1394] = "A6FFA6", -- party
			[1396] = "FF7038", -- raid
			[9087] = "404080", -- guardian
			[131072] = "CC54D9", -- focus
			[20334] = "FF75FF", -- enemy players/npcs
			[3864] = "FFFFFF", -- outsider
		},
		spellColors = {
			[SCHOOL_MASK_PHYSICAL]						= "FFFFFF",
			[SCHOOL_MASK_HOLY]							= "FFFF4D",
			[SCHOOL_MASK_FIRE]							= "FF262E",
			[SCHOOL_MASK_NATURE]						= "66FF66",
			[SCHOOL_MASK_FROST]							= "4D4DE6",
			[SCHOOL_MASK_SHADOW]						= "FFB3FF",
			[SCHOOL_MASK_ARCANE]						= "BFBFBF",
			-- Physical and a Magical
			[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FIRE]	= "FF9397",
			[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FROST]	= "A6A6F3",
			[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_ARCANE]	= "DFDFDF",
			[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_NATURE]	= "B3FFB3",
			[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_SHADOW]	= "FFD9FF",
			[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_HOLY]	= "FFFFD3",
			-- Two Magical Schools
			[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST]		= "A63A8A",
			[SCHOOL_MASK_FIRE + SCHOOL_MASK_ARCANE]		= "DF7377",
			[SCHOOL_MASK_FIRE + SCHOOL_MASK_NATURE]		= "B3934A",
			[SCHOOL_MASK_FIRE + SCHOOL_MASK_SHADOW]		= "FF6D97",
			[SCHOOL_MASK_FIRE + SCHOOL_MASK_HOLY]		= "FF933E",
			[SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE]	= "8686D3",
			[SCHOOL_MASK_FROST + SCHOOL_MASK_NATURE]	= "5AA6A6",
			[SCHOOL_MASK_FROST + SCHOOL_MASK_SHADOW]	= "A680F3",
			[SCHOOL_MASK_FROST + SCHOOL_MASK_HOLY]		= "A6A69A",
			[SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE]	= "93DF93",
			[SCHOOL_MASK_ARCANE + SCHOOL_MASK_SHADOW]	= "DFB9DF",
			[SCHOOL_MASK_ARCANE + SCHOOL_MASK_HOLY]		= "DFDF86",
			[SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW]	= "B3D9B3",
			[SCHOOL_MASK_NATURE + SCHOOL_MASK_HOLY]		= "B3FF5A",
			[SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY]		= "FFD9A6",
			-- Three or more schools
			[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_NATURE]																						= "917B7E",
			[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW]											= "B094A5",
			[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY]						= "BDA696",
			[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY]	= "C7B3A5"
		},
		otherColors = {
			heal = "FFFF4D",
			miss = "FF7F7F",
			buff = "33FF33",
			debuff = "FF3333",
			skill = "FFFF66",
			timestampColor = "979797",
			
		},
		blacklist = {},
		watches = {},
		braces = {
			preTimestamp = "[",
			postTimestamp = "]",
			preSource = "[",
			postSource = "]",
			preSpell = "[",
			postSpell = "]",
			preDest = "[",
			postDest = "]",
			preAmount = "[",
			postAmount = "]",
			preAmountDots = "~",
			postAmountDots = "~",
			preSchoolText = "[",
			postSchoolText = "]",
			preCrit = "*",
			postCrit = "*",
		},
		braceColors = {
			Timestamp = "979797",
			Source = "FF75FF",
			Spell = "FF75FF",
			Dest = "FF75FF",
			Amount = "FF75FF",
			AmountDots = "FF75FF",
			SchoolText = "FF75FF",
		},
		
		contextMenu = true,
		quickButtons = true,
		
		timestamp = true,
		colorTimestamp = true,
		braceTimestamp = true,
		colorTimestampBrace = true,
		timestampFormat = "%X",
		
		colorUnitByClass = false,
		
		braceSrcUnit = true,
		colorSrc = true,
		colorSrcBraces = false,
		
		prefix = true,
		colorPrefix = true,
		["++"] = "++",
		["--"] = "--",
		["~~"] = "~~",
		
		braceSpell = false,
		colorSpell = true,
		colorSpellBraces = false,
		
		suffixString = true,
		resultInSuffix = true,
		colorSuffix = true,
		
		braceDestUnit = true,
		colorDest = true,
		colorDestBraces = false,
		
		braceAmount = false,
		colorAmount = true,
		colorAmountBraces = false,
		braceAmountDots = true,
		colorAmountDotBraces = false,
		markCrits = true,
		
		overhealing = true,
		overkill = true,
		
		resultString = true,
		
		powerType = true,
		
		schoolText = false,
		colorSchoolText = true,
		braceSchoolText = false,
		colorSchoolTextBraces = false,
		
		format = {
			ABSORB = "(A%d)",
			BLOCK = BLOCK,
			BLOCK_VALUE = "(B%d)",
			CRITICAL = L["Crit"],
			CRUSHING = L["Crush"],
			DEFLECT = DEFLECT,
			DODGE = DODGE,
			EVADE = L["Evade"],
			GLANCING = L["Glancing"],
			IMMUNE = SPELL_FAILED_IMMUNE,
			MISS = MISS,
			PARRY = PARRY,
			REFLECT = REFLECT,
			RESIST = "(R%d)",
			VULNERABILITY = "(V%d)",
		},
		filters = {
			[1] = {
				name = "By me",
				active = true,
				ENVIRONMENTAL_DAMAGE = true,
				SWING_DAMAGE = true,
				SWING_MISSED = true,
				RANGE_DAMAGE = true,
				RANGE_MISSED = true,
				SPELL_CAST_START = false,
				SPELL_CAST_SUCCESS = false,
				SPELL_CAST_FAILED = false,
				SPELL_MISSED = true,
				SPELL_DAMAGE = true,
				SPELL_HEAL = true,
				SPELL_ENERGIZE = true,
				SPELL_DRAIN = true,
				SPELL_LEECH = true,
				SPELL_SUMMON = true,
				SPELL_RESURRECT = true,
				SPELL_CREATE = true,
				SPELL_INSTAKILL = true,
				SPELL_INTERRUPT = true,
				SPELL_EXTRA_ATTACKS = true,
				SPELL_DURABILITY_DAMAGE = false,
				SPELL_DURABILITY_DAMAGE_ALL = false,
				SPELL_AURA_APPLIED = true,
				SPELL_AURA_APPLIED_DOSE = true,
				SPELL_AURA_REMOVED = true,
				SPELL_AURA_REMOVED_DOSE = true,
				SPELL_AURA_BROKEN = true,
				SPELL_AURA_BROKEN_SPELL = true,
				SPELL_AURA_REFRESH = true,
				SPELL_DISPEL = true,
				SPELL_STOLEN = true,
				ENCHANT_APPLIED = true,
				ENCHANT_REMOVED = true,
				SPELL_PERIODIC_MISSED = true,
				SPELL_PERIODIC_DAMAGE = true,
				SPELL_PERIODIC_HEAL = true,
				SPELL_PERIODIC_ENERGIZE = true,
				SPELL_PERIODIC_DRAIN = true,
				SPELL_PERIODIC_LEECH = true,
				SPELL_DISPEL_FAILED = true,
				DAMAGE_SHIELD = true,
				DAMAGE_SHIELD_MISSED = true,
				DAMAGE_SPLIT = true,
				PARTY_KILL = true,
				UNIT_DIED = true,
				UNIT_DESTROYED = true,
				SPELL_BUILDING_DAMAGE = true,
				SPELL_BUILDING_HEAL = true,
				UNIT_DISSIPATES = true,
				sourceUnit = {
					--Controller affiliation
					COMBATLOG_OBJECT_AFFILIATION_MINE = true,
					COMBATLOG_OBJECT_AFFILIATION_PARTY = false,
					COMBATLOG_OBJECT_AFFILIATION_RAID = false,
					COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = false,
					--Reaction
					COMBATLOG_OBJECT_REACTION_FRIENDLY = true,
					COMBATLOG_OBJECT_REACTION_NEUTRAL = false,
					COMBATLOG_OBJECT_REACTION_HOSTILE = false,
					--Controller
					COMBATLOG_OBJECT_CONTROL_PLAYER = true,
					COMBATLOG_OBJECT_CONTROL_NPC = false,
					--Type
					COMBATLOG_OBJECT_TYPE_PLAYER = true,
					COMBATLOG_OBJECT_TYPE_NPC = false,
					COMBATLOG_OBJECT_TYPE_PET = true,
					COMBATLOG_OBJECT_TYPE_GUARDIAN = true,
					COMBATLOG_OBJECT_TYPE_OBJECT = false,
					--Special cases (non-exclusive)
					COMBATLOG_OBJECT_TARGET = false,
					COMBATLOG_OBJECT_FOCUS = false,
					COMBATLOG_OBJECT_MAINTANK = false,
					COMBATLOG_OBJECT_MAINASSIST = false,
					COMBATLOG_OBJECT_RAIDTARGET1 = false,
					COMBATLOG_OBJECT_RAIDTARGET2 = false,
					COMBATLOG_OBJECT_RAIDTARGET3 = false,
					COMBATLOG_OBJECT_RAIDTARGET4 = false,
					COMBATLOG_OBJECT_RAIDTARGET5 = false,
					COMBATLOG_OBJECT_RAIDTARGET6 = false,
					COMBATLOG_OBJECT_RAIDTARGET7 = false,
					COMBATLOG_OBJECT_RAIDTARGET8 = false,
					COMBATLOG_OBJECT_NONE = false,
				},
				destUnit = {
					--Controller affiliation
					COMBATLOG_OBJECT_AFFILIATION_MINE = false,
					COMBATLOG_OBJECT_AFFILIATION_PARTY = false,
					COMBATLOG_OBJECT_AFFILIATION_RAID = false,
					COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = false,
					--Reaction
					COMBATLOG_OBJECT_REACTION_FRIENDLY = false,
					COMBATLOG_OBJECT_REACTION_NEUTRAL = false,
					COMBATLOG_OBJECT_REACTION_HOSTILE = false,
					--Controller
					COMBATLOG_OBJECT_CONTROL_PLAYER = false,
					COMBATLOG_OBJECT_CONTROL_NPC = false,
					--Type
					COMBATLOG_OBJECT_TYPE_PLAYER = false,
					COMBATLOG_OBJECT_TYPE_NPC = false,
					COMBATLOG_OBJECT_TYPE_PET = false,
					COMBATLOG_OBJECT_TYPE_GUARDIAN = false,
					COMBATLOG_OBJECT_TYPE_OBJECT = false,
					--Special cases (non-exclusive)
					COMBATLOG_OBJECT_TARGET = false,
					COMBATLOG_OBJECT_FOCUS = false,
					COMBATLOG_OBJECT_MAINTANK = false,
					COMBATLOG_OBJECT_MAINASSIST = false,
					COMBATLOG_OBJECT_RAIDTARGET1 = false,
					COMBATLOG_OBJECT_RAIDTARGET2 = false,
					COMBATLOG_OBJECT_RAIDTARGET3 = false,
					COMBATLOG_OBJECT_RAIDTARGET4 = false,
					COMBATLOG_OBJECT_RAIDTARGET5 = false,
					COMBATLOG_OBJECT_RAIDTARGET6 = false,
					COMBATLOG_OBJECT_RAIDTARGET7 = false,
					COMBATLOG_OBJECT_RAIDTARGET8 = false,
					COMBATLOG_OBJECT_NONE = false,
				},
			},
			[2] = {
				name = "To me",
				active = true,
				ENVIRONMENTAL_DAMAGE = true,
				SWING_DAMAGE = true,
				SWING_MISSED = true,
				RANGE_DAMAGE = true,
				RANGE_MISSED = true,
				SPELL_CAST_START = false,
				SPELL_CAST_SUCCESS = false,
				SPELL_CAST_FAILED = false,
				SPELL_MISSED = true,
				SPELL_DAMAGE = true,
				SPELL_HEAL = true,
				SPELL_ENERGIZE = true,
				SPELL_DRAIN = true,
				SPELL_LEECH = true,
				SPELL_SUMMON = true,
				SPELL_RESURRECT = true,
				SPELL_CREATE = true,
				SPELL_INSTAKILL = true,
				SPELL_INTERRUPT = true,
				SPELL_EXTRA_ATTACKS = true,
				SPELL_DURABILITY_DAMAGE = false,
				SPELL_DURABILITY_DAMAGE_ALL = false,
				SPELL_AURA_APPLIED = true,
				SPELL_AURA_APPLIED_DOSE = true,
				SPELL_AURA_REMOVED = true,
				SPELL_AURA_REMOVED_DOSE = true,
				SPELL_AURA_BROKEN = true,
				SPELL_AURA_BROKEN_SPELL = true,
				SPELL_AURA_REFRESH = true,
				SPELL_DISPEL = true,
				SPELL_STOLEN = true,
				ENCHANT_APPLIED = true,
				ENCHANT_REMOVED = true,
				SPELL_PERIODIC_MISSED = true,
				SPELL_PERIODIC_DAMAGE = true,
				SPELL_PERIODIC_HEAL = true,
				SPELL_PERIODIC_ENERGIZE = true,
				SPELL_PERIODIC_DRAIN = true,
				SPELL_PERIODIC_LEECH = true,
				SPELL_DISPEL_FAILED = true,
				DAMAGE_SHIELD = true,
				DAMAGE_SHIELD_MISSED = true,
				DAMAGE_SPLIT = true,
				PARTY_KILL = true,
				UNIT_DIED = true,
				UNIT_DESTROYED = true,
				SPELL_BUILDING_DAMAGE = true,
				SPELL_BUILDING_HEAL = true,
				UNIT_DISSIPATES = true,
				sourceUnit = {
					--Controller affiliation
					COMBATLOG_OBJECT_AFFILIATION_MINE = false,
					COMBATLOG_OBJECT_AFFILIATION_PARTY = false,
					COMBATLOG_OBJECT_AFFILIATION_RAID = false,
					COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = false,
					--Reaction
					COMBATLOG_OBJECT_REACTION_FRIENDLY = false,
					COMBATLOG_OBJECT_REACTION_NEUTRAL = false,
					COMBATLOG_OBJECT_REACTION_HOSTILE = false,
					--Controller
					COMBATLOG_OBJECT_CONTROL_PLAYER = false,
					COMBATLOG_OBJECT_CONTROL_NPC = false,
					--Type
					COMBATLOG_OBJECT_TYPE_PLAYER = false,
					COMBATLOG_OBJECT_TYPE_NPC = false,
					COMBATLOG_OBJECT_TYPE_PET = false,
					COMBATLOG_OBJECT_TYPE_GUARDIAN = false,
					COMBATLOG_OBJECT_TYPE_OBJECT = false,
					--Special cases (non-exclusive)
					COMBATLOG_OBJECT_TARGET = false,
					COMBATLOG_OBJECT_FOCUS = false,
					COMBATLOG_OBJECT_MAINTANK = false,
					COMBATLOG_OBJECT_MAINASSIST = false,
					COMBATLOG_OBJECT_RAIDTARGET1 = false,
					COMBATLOG_OBJECT_RAIDTARGET2 = false,
					COMBATLOG_OBJECT_RAIDTARGET3 = false,
					COMBATLOG_OBJECT_RAIDTARGET4 = false,
					COMBATLOG_OBJECT_RAIDTARGET5 = false,
					COMBATLOG_OBJECT_RAIDTARGET6 = false,
					COMBATLOG_OBJECT_RAIDTARGET7 = false,
					COMBATLOG_OBJECT_RAIDTARGET8 = false,
					COMBATLOG_OBJECT_NONE = false,
				},
				destUnit = {
					--Controller affiliation
					COMBATLOG_OBJECT_AFFILIATION_MINE = true,
					COMBATLOG_OBJECT_AFFILIATION_PARTY = false,
					COMBATLOG_OBJECT_AFFILIATION_RAID = false,
					COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = false,
					--Reaction
					COMBATLOG_OBJECT_REACTION_FRIENDLY = true,
					COMBATLOG_OBJECT_REACTION_NEUTRAL = false,
					COMBATLOG_OBJECT_REACTION_HOSTILE = false,
					--Controller
					COMBATLOG_OBJECT_CONTROL_PLAYER = true,
					COMBATLOG_OBJECT_CONTROL_NPC = false,
					--Type
					COMBATLOG_OBJECT_TYPE_PLAYER = true,
					COMBATLOG_OBJECT_TYPE_NPC = false,
					COMBATLOG_OBJECT_TYPE_PET = true,
					COMBATLOG_OBJECT_TYPE_GUARDIAN = true,
					COMBATLOG_OBJECT_TYPE_OBJECT = false,
					--Special cases (non-exclusive)
					COMBATLOG_OBJECT_TARGET = false,
					COMBATLOG_OBJECT_FOCUS = false,
					COMBATLOG_OBJECT_MAINTANK = false,
					COMBATLOG_OBJECT_MAINASSIST = false,
					COMBATLOG_OBJECT_RAIDTARGET1 = false,
					COMBATLOG_OBJECT_RAIDTARGET2 = false,
					COMBATLOG_OBJECT_RAIDTARGET3 = false,
					COMBATLOG_OBJECT_RAIDTARGET4 = false,
					COMBATLOG_OBJECT_RAIDTARGET5 = false,
					COMBATLOG_OBJECT_RAIDTARGET6 = false,
					COMBATLOG_OBJECT_RAIDTARGET7 = false,
					COMBATLOG_OBJECT_RAIDTARGET8 = false,
					COMBATLOG_OBJECT_NONE = false,
				},
			},
			[3] = {
				name = "Everything",
				active = false,
				ENVIRONMENTAL_DAMAGE = true,
				SWING_DAMAGE = true,
				SWING_MISSED = true,
				RANGE_DAMAGE = true,
				RANGE_MISSED = true,
				SPELL_CAST_START = true,
				SPELL_CAST_SUCCESS = true,
				SPELL_CAST_FAILED = true,
				SPELL_MISSED = true,
				SPELL_DAMAGE = true,
				SPELL_HEAL = true,
				SPELL_ENERGIZE = true,
				SPELL_DRAIN = true,
				SPELL_LEECH = true,
				SPELL_SUMMON = true,
				SPELL_RESURRECT = true,
				SPELL_CREATE = true,
				SPELL_INSTAKILL = true,
				SPELL_INTERRUPT = true,
				SPELL_EXTRA_ATTACKS = true,
				SPELL_DURABILITY_DAMAGE = true,
				SPELL_DURABILITY_DAMAGE_ALL = true,
				SPELL_AURA_APPLIED = true,
				SPELL_AURA_APPLIED_DOSE = true,
				SPELL_AURA_REMOVED = true,
				SPELL_AURA_REMOVED_DOSE = true,
				SPELL_AURA_BROKEN = true,
				SPELL_AURA_BROKEN_SPELL = true,
				SPELL_AURA_REFRESH = true,
				SPELL_DISPEL = true,
				SPELL_STOLEN = true,
				ENCHANT_APPLIED = true,
				ENCHANT_REMOVED = true,
				SPELL_PERIODIC_MISSED = true,
				SPELL_PERIODIC_DAMAGE = true,
				SPELL_PERIODIC_HEAL = true,
				SPELL_PERIODIC_ENERGIZE = true,
				SPELL_PERIODIC_DRAIN = true,
				SPELL_PERIODIC_LEECH = true,
				SPELL_DISPEL_FAILED = true,
				DAMAGE_SHIELD = true,
				DAMAGE_SHIELD_MISSED = true,
				DAMAGE_SPLIT = true,
				PARTY_KILL = true,
				UNIT_DIED = true,
				UNIT_DESTROYED = true,
				SPELL_BUILDING_DAMAGE = true,
				SPELL_BUILDING_HEAL = true,
				UNIT_DISSIPATES = true,
				sourceUnit = {
					--Controller affiliation
					COMBATLOG_OBJECT_AFFILIATION_MINE = true,
					COMBATLOG_OBJECT_AFFILIATION_PARTY = true,
					COMBATLOG_OBJECT_AFFILIATION_RAID = true,
					COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = true,
					--Reaction
					COMBATLOG_OBJECT_REACTION_FRIENDLY = true,
					COMBATLOG_OBJECT_REACTION_NEUTRAL = true,
					COMBATLOG_OBJECT_REACTION_HOSTILE = true,
					--Controller
					COMBATLOG_OBJECT_CONTROL_PLAYER = true,
					COMBATLOG_OBJECT_CONTROL_NPC = true,
					--Type
					COMBATLOG_OBJECT_TYPE_PLAYER = true,
					COMBATLOG_OBJECT_TYPE_NPC = true,
					COMBATLOG_OBJECT_TYPE_PET = true,
					COMBATLOG_OBJECT_TYPE_GUARDIAN = true,
					COMBATLOG_OBJECT_TYPE_OBJECT = true,
					--Special cases (non-exclusive)
					COMBATLOG_OBJECT_TARGET = true,
					COMBATLOG_OBJECT_FOCUS = true,
					COMBATLOG_OBJECT_MAINTANK = true,
					COMBATLOG_OBJECT_MAINASSIST = true,
					COMBATLOG_OBJECT_RAIDTARGET1 = true,
					COMBATLOG_OBJECT_RAIDTARGET2 = true,
					COMBATLOG_OBJECT_RAIDTARGET3 = true,
					COMBATLOG_OBJECT_RAIDTARGET4 = true,
					COMBATLOG_OBJECT_RAIDTARGET5 = true,
					COMBATLOG_OBJECT_RAIDTARGET6 = true,
					COMBATLOG_OBJECT_RAIDTARGET7 = true,
					COMBATLOG_OBJECT_RAIDTARGET8 = true,
					COMBATLOG_OBJECT_NONE = true,
				},
				destUnit = {
					--Controller affiliation
					COMBATLOG_OBJECT_AFFILIATION_MINE = true,
					COMBATLOG_OBJECT_AFFILIATION_PARTY = true,
					COMBATLOG_OBJECT_AFFILIATION_RAID = true,
					COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = true,
					--Reaction
					COMBATLOG_OBJECT_REACTION_FRIENDLY = true,
					COMBATLOG_OBJECT_REACTION_NEUTRAL = true,
					COMBATLOG_OBJECT_REACTION_HOSTILE = true,
					--Controller
					COMBATLOG_OBJECT_CONTROL_PLAYER = true,
					COMBATLOG_OBJECT_CONTROL_NPC = true,
					--Type
					COMBATLOG_OBJECT_TYPE_PLAYER = true,
					COMBATLOG_OBJECT_TYPE_NPC = true,
					COMBATLOG_OBJECT_TYPE_PET = true,
					COMBATLOG_OBJECT_TYPE_GUARDIAN = true,
					COMBATLOG_OBJECT_TYPE_OBJECT = true,
					--Special cases (non-exclusive)
					COMBATLOG_OBJECT_TARGET = true,
					COMBATLOG_OBJECT_FOCUS = true,
					COMBATLOG_OBJECT_MAINTANK = true,
					COMBATLOG_OBJECT_MAINASSIST = true,
					COMBATLOG_OBJECT_RAIDTARGET1 = true,
					COMBATLOG_OBJECT_RAIDTARGET2 = true,
					COMBATLOG_OBJECT_RAIDTARGET3 = true,
					COMBATLOG_OBJECT_RAIDTARGET4 = true,
					COMBATLOG_OBJECT_RAIDTARGET5 = true,
					COMBATLOG_OBJECT_RAIDTARGET6 = true,
					COMBATLOG_OBJECT_RAIDTARGET7 = true,
					COMBATLOG_OBJECT_RAIDTARGET8 = true,
					COMBATLOG_OBJECT_NONE = true,
				},
			},
		},
	}
}

function YurrCombatLog:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("YurrCombatLogDB", defaults, true)
	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
	self:RegisterChatCommand("yurrcombatlog", "showoptionsgui")
	self:RegisterChatCommand("ycl", "showoptionsgui")
	
	if self.db.profile.version < 0.66 then
		--self.db:ResetDB("Default")
		self.db:ResetProfile()
	end
	self.db.profile.version = tonumber(GetAddOnMetadata("YurrCombatLog", "Version"))
	
	self:InitQuickButtons()
	
	prefixList = {
		["_AURA_APPLIED"] = self.db.profile["++"],
		["_AURA_APPLIED_DOSE"] = self.db.profile["++"],
		["_AURA_REMOVED"] = self.db.profile["--"],
		["_AURA_REMOVED_DOSE"] = self.db.profile["--"],
		["_AURA_REFRESH"] = self.db.profile["~~"],
		["_CAST_START"] = L.begins,
		["_CAST_SUCCESS"] = L.casts,
		["_CREATE"] = L.create,
		["_INSTAKILL"] = L.instantkill,
		["_AURA_BROKEN"] = L.broke,
		["_AURA_BROKEN_SPELL"] = L.broke
	}
	suffixList = {
		["_HEAL"] = L.Heal,
		["_PERIODIC_HEAL"] = L.Regen,
		["_PERIODIC_DAMAGE"] = L.Dot,
		["_PERIODIC_MISSED"] = L.Dot,
		["_DISPEL"] = L.Dispel,
		["_DISPEL_FAILED"] = L.Dispel,
		["_INTERRUPT"] = L.Interrupt,
		["_SUMMON"] = L.Summon,
		["_RESURRECT"] = L.Resurrect,
		["MISS"] = self.db.profile.format.MISS
	}
	spellList = {
		["_DISPEL"] = self.db.profile.spellColors[school],
		["_AURA_REMOVED"] = self.db.profile.otherColors.skill,
		["_AURA_REMOVED_DOSE"] = self.db.profile.otherColors.skill,
		["_MISSED"] = self.db.profile.otherColors.miss,
		["BUFF"] = self.db.profile.otherColors.buff,
		["DEBUFF"] = self.db.profile.otherColors.debuff,
		["_HEAL"] = self.db.profile.otherColors.heal,
		["_PERIODIC_HEAL"] = self.db.profile.otherColors.heal
	}
	
	local function RGBPercToHex(r, g, b)
		r = r <= 1 and r >= 0 and r or 0
		g = g <= 1 and g >= 0 and g or 0
		b = b <= 1 and b >= 0 and b or 0
		return string.format("%02x%02x%02x", r*255, g*255, b*255)
	end
	for class, color in pairs(RAID_CLASS_COLORS) do
		classColors[class] = RGBPercToHex(color.r, color.g, color.b)
	end
end

function YurrCombatLog:RefreshConfig()
	self:ReInitQuickButtons()
	self.db.profile.version = tonumber(GetAddOnMetadata("YurrCombatLog", "Version"))
end

function YurrCombatLog:InitQuickButtons()
	local function updateGTT(QuickButton, k, v)
		GameTooltip:SetOwner(QuickButton, "ANCHOR_TOP")
		local state = self.db.profile.filters[k].active
		local enable
		if not state then
			enable = L.on
		else
			enable = L.off
		end
		GameTooltip:SetText(format(L.clickToToggle, v.name, enable))
		GameTooltip:Show()
	end
	
	CreateFrame("Frame", "YCLQuickButtonFrame")
	YCLQuickButtonFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background"})
	YCLQuickButtonFrame:SetBackdropColor(0, 0, 0)
	YCLQuickButtonFrame:SetParent(_G[format("%sTab", self.db.profile.outputFrame)])
	YCLQuickButtonFrame:SetPoint("TOPLEFT", _G[format("%sTab", self.db.profile.outputFrame, "Tab")], "TOPRIGHT", 0, -13)
	YCLQuickButtonFrame:SetPoint("BOTTOMRIGHT", _G[self.db.profile.outputFrame], "TOPRIGHT", 0, 5)
	if not self.db.profile.quickButtons then YCLQuickButtonFrame:Hide() end
	
	local width = 0
	local limit = YCLQuickButtonFrame:GetWidth() - 15
	local dropdownStart
	for k,v in ipairs(self.db.profile.filters) do
		if v then
			if width >= limit then
				dropdownStart = k
				return
			end
			local QuickButton = CreateFrame("Button")
			
			QuickButtonText = QuickButton:CreateFontString("QuickButton")
			QuickButtonText:SetFont("Fonts\\FRIZQT__.TTF", 11)
			QuickButtonText:SetText(v.name)
			QuickButtonText:SetPoint("CENTER", QuickButton)
			
			QuickButton:SetPoint("TOPLEFT", YCLQuickButtonFrame, "TOPLEFT", width + 8, 0)
			
			width = width + QuickButtonText:GetWidth() + 8
			
			QuickButton:SetPoint("BOTTOMRIGHT", YCLQuickButtonFrame, "BOTTOMLEFT", width + 8, 2)
			QuickButton:SetScript("OnClick", function()
				for k2,v2 in ipairs(self.db.profile.filters) do
					if v2 then
						if v.name == v2.name then
							v2.active = not v2.active
						end
					end
				end
				updateGTT(QuickButton, k, v)
				self:setFilter()
			end)
			QuickButton:SetScript("OnEnter", function()
				updateGTT(QuickButton, k, v)
			end)
			QuickButton:SetScript("OnLeave", function() GameTooltip:Hide() end)
			QuickButton:SetParent(YCLQuickButtonFrame)
			QuickButtonText:SetParent(QuickButton)
			
			if k == #self.db.profile.filters then return end
		end
	end
end
function YurrCombatLog:ReInitQuickButtons()
	if YCLQuickButtonFrame then
		--local oldF = YCLQuickButtonFrame
		--oldF:Hide()
		YCLQuickButtonFrame:Hide()
		YCLQuickButtonFrame = nil
	end
	self:InitQuickButtons()
end

local s = false
function YurrCombatLog:showoptionsgui()
	if not IsAddOnLoaded("YurrCombatLogOptions") then
		local _, reason = LoadAddOn("YurrCombatLogOptions")
		local _, title = GetAddOnInfo("YurrCombatLogOptions")
		if (not title) then title = "YurrCombatLogOptions" end
		if reason then
			self:Print(format("%s (Enable) LoadOnDemand Error - %s", title, reason))
		end
    end
	if s then
		LibStub("AceConfigDialog-3.0"):Close("YurrCombatLog")
	else
		LibStub("AceConfigDialog-3.0"):Open("YurrCombatLog")
	end
	s = not s
end

local ldb = LibStub("LibDataBroker-1.1")
local dataobj = ldb:NewDataObject("YurrCombatLogOptions", {type = "launcher", icon = "Interface\\Icons\\Ability_MeleeDamage", text = "YurrCombatLog"})

function dataobj.OnTooltipShow(self)
	self:AddLine(dataobj.text)
	self:AddLine(format("|cffffff00%s", L.leftClick))
end
dataobj.OnClick = function() YurrCombatLog:showoptionsgui() end

--[[
[SCHOOL_MASK_PHYSICAL]						= STRING_SCHOOL_PHYSICAL,
[SCHOOL_MASK_HOLY]							= STRING_SCHOOL_HOLY,
[SCHOOL_MASK_FIRE]							= STRING_SCHOOL_FIRE,
[SCHOOL_MASK_NATURE]						= STRING_SCHOOL_NATURE,
[SCHOOL_MASK_FROST]							= STRING_SCHOOL_FROST,
[SCHOOL_MASK_SHADOW]						= STRING_SCHOOL_SHADOW,
[SCHOOL_MASK_ARCANE]						= STRING_SCHOOL_ARCANE,
-- Physical and a Magical
[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FIRE]	= STRING_SCHOOL_FLAMESTRIKE,
[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FROST]	= STRING_SCHOOL_FROSTSTRIKE,
[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_ARCANE]	= STRING_SCHOOL_SPELLSTRIKE,
[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_NATURE]	= STRING_SCHOOL_STORMSTRIKE,
[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_SHADOW]	= STRING_SCHOOL_SHADOWSTRIKE,
[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_HOLY]	= STRING_SCHOOL_HOLYSTRIKE,
-- Two Magical Schools
[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST]		= STRING_SCHOOL_FROSTFIRE,
[SCHOOL_MASK_FIRE + SCHOOL_MASK_ARCANE]		= STRING_SCHOOL_SPELLFIRE,
[SCHOOL_MASK_FIRE + SCHOOL_MASK_NATURE]		= STRING_SCHOOL_FIRESTORM,
[SCHOOL_MASK_FIRE + SCHOOL_MASK_SHADOW]		= STRING_SCHOOL_SHADOWFLAME,
[SCHOOL_MASK_FIRE + SCHOOL_MASK_HOLY]		= STRING_SCHOOL_HOLYFIRE,
[SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE]	= STRING_SCHOOL_SPELLFROST,
[SCHOOL_MASK_FROST + SCHOOL_MASK_NATURE]	= STRING_SCHOOL_FROSTSTORM,
[SCHOOL_MASK_FROST + SCHOOL_MASK_SHADOW]	= STRING_SCHOOL_SHADOWFROST,
[SCHOOL_MASK_FROST + SCHOOL_MASK_HOLY]		= STRING_SCHOOL_HOLYFROST,
[SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE]	= STRING_SCHOOL_SPELLSTORM,
[SCHOOL_MASK_ARCANE + SCHOOL_MASK_SHADOW]	= STRING_SCHOOL_SPELLSHADOW,
[SCHOOL_MASK_ARCANE + SCHOOL_MASK_HOLY]		= STRING_SCHOOL_DIVINE,
[SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW]	= STRING_SCHOOL_SHADOWSTORM,
[SCHOOL_MASK_NATURE + SCHOOL_MASK_HOLY]		= STRING_SCHOOL_HOLYSTORM,
[SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY]		= STRING_SCHOOL_SHADOWLIGHT,
-- Three or more schools
[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_NATURE]																						= STRING_SCHOOL_ELEMENTAL,
[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW]											= STRING_SCHOOL_CHROMATIC,
[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY]						= STRING_SCHOOL_MAGIC,
[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY]	= STRING_SCHOOL_CHAOS,]]

local orgFCF_DockUpdate = FCF_DockUpdate
function YurrCombatLog:OnEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT")
	
	self:RawHook("Blizzard_CombatLog_Refilter", true)
	self:RawHook("Blizzard_CombatLog_RefilterUpdate", true)
	self:RawHook("CombatLog_AddEvent", true)
	self:RawHook("Blizzard_CombatLog_ApplyFilters", "setFilter", true)
	self:RawHook("FCF_DockUpdate", true)
	self:HookScript(CombatLogQuickButtonFrame_Custom, "OnShow", "CombatLogQuickButtonFrame_Custom_OnShow")
	
	CombatLogQuickButtonFrameProgressBar = _G.CombatLogQuickButtonFrame_CustomProgressBar
	
	if self.db.profile.contextMenu then self:InsertContext() end
end

-- ninjad from Prat_ArmoryLink
local popupList
function YurrCombatLog:UnitPopup_Click(info)
	local name = UIDROPDOWNMENU_INIT_MENU.name
	local button = info.value
	local click = {
		["ADD_TO_YCL"] = function()
			self.db.profile.watches[name] = UnitGUID(name)
			if IsAddOnLoaded("YurrCombatLogOptions") then
				local module = self:GetModule("YurrCombatLogOptions")
				local options = AceConfigRegistry:GetOptionsTable("YurrCombatLog", "dialog", "YurrCombatLog-0.6")
				options.args.watches.args[name] = module:createWatch(name, UnitGUID(name))
				AceConfigRegistry:NotifyChange("YurrCombatLog")
			end
			self:setFilter()
		end,
		["REMOVE_FROM_YCL"] = function()
			self.db.profile.watches[name] = nil
			if IsAddOnLoaded("YurrCombatLogOptions") then
				local module = self:GetModule("YurrCombatLogOptions")
				local options = AceConfigRegistry:GetOptionsTable("YurrCombatLog", "dialog", "YurrCombatLog-0.6")
				options.args.watches.args[name] = nil
				AceConfigRegistry:NotifyChange("YurrCombatLog")
			end
			self:setFilter()
		end
	}
	if click[button] then click[button]() end
end
function YurrCombatLog:UnitPopup_ShowMenu(dropdownMenu, which, unit, name, userData, ...)
	for i=1, UIDROPDOWNMENU_MAXBUTTONS do
		button = _G[format("DropDownList%dButton%d", UIDROPDOWNMENU_MENU_LEVEL, i)];

		-- Patch our handler function back in
		if  button.value == "ADD_TO_YCL" then
		    button.func = UnitPopupButtons["ADD_TO_YCL"].func
		elseif  button.value == "REMOVE_FROM_YCL" then
		    button.func = UnitPopupButtons["REMOVE_FROM_YCL"].func
		end
	end
end
function YurrCombatLog:InsertContext()
	-- ninjad from Prat_ArmoryLink
	self:SecureHook("UnitPopup_ShowMenu")
	UnitPopupButtons['ADD_TO_YCL'] = { 
		text = L["Watch in (Yurr)CombatLog"], 
		dist = 0, 
		func = function(info) YurrCombatLog:UnitPopup_Click(info) end,
	}
	UnitPopupButtons['REMOVE_FROM_YCL'] = { 
		text = L["Remove watch from (Yurr)CombatLog"],
		dist = 0, 
		func = function(info) YurrCombatLog:UnitPopup_Click(info) end,
	}
	popupList = { 
		'PARTY',        -- party frame.
		'FRIEND',       -- friend frame
		'SELF',         -- ?
		'PLAYER',       -- ?
		--'RAID',         -- raid frames
		'RAID_PLAYER',  -- player's raid frame?
		'TEAM'          -- arena team
	}
	for key,val in pairs(popupList) do
		tinsert(UnitPopupMenus[val], #UnitPopupMenus[val]-1,'ADD_TO_YCL')
		tinsert(UnitPopupMenus[val], #UnitPopupMenus[val]-1,'REMOVE_FROM_YCL')
	end
end
function YurrCombatLog:RemoveContext()

	for j = 1, #popupList do
		local t = popupList[j]
		for i = 1, #UnitPopupMenus[t] do
			if UnitPopupMenus[t][i] == "ADD_TO_YCL" then
				tremove(UnitPopupMenus[t], i)
				break
			end
		end
	end
	for j = 1, #popupList do
		local t = popupList[j]
		for i = 1, #UnitPopupMenus[t] do
			if UnitPopupMenus[t][i] == "REMOVE_FROM_YCL" then
				tremove(UnitPopupMenus[t], i)
				break
			end
		end
	end
	self:Unhook("UnitPopup_ShowMenu")
end

function YurrCombatLog:Blizzard_CombatLog_Refilter()
	local count = CombatLogGetNumEntries();
	
	COMBATLOG:SetMaxLines(COMBATLOG_MESSAGE_LIMIT);
	_G[self.db.profile.outputFrame]:SetMaxLines(COMBATLOG_MESSAGE_LIMIT)

	count = max(1, min(count, COMBATLOG_MESSAGE_LIMIT));

	CombatLogSetCurrentEntry(0);
	
	--COMBATLOG:Clear();
	_G[self.db.profile.outputFrame]:Clear()
	
	CombatLogQuickButtonFrameProgressBar:SetMinMaxValues(0, count);	
	CombatLogQuickButtonFrameProgressBar:SetValue(0);
	CombatLogQuickButtonFrameProgressBar:Show();

	CombatLogUpdateFrame.refiltering = true;
	CombatLogUpdateFrame:SetScript("OnUpdate", Blizzard_CombatLog_RefilterUpdate)	
	
	Blizzard_CombatLog_RefilterUpdate()
end
function YurrCombatLog:Blizzard_CombatLog_RefilterUpdate()
	local valid = CombatLogGetCurrentEntry();
	
	local total = 0;
	while (valid and total < COMBATLOG_LIMIT_PER_FRAME) do 
		--local text, r, g, b, a = CombatLog_OnEvent(Blizzard_CombatLog_CurrentSettings, CombatLogGetCurrentEntry());
		local YCLtext = self:processEvent("COMBAT_LOG_EVENT", CombatLogGetCurrentEntry())
		--COMBATLOG:AddMessage( text, r, g, b, nil, true );
		_G[self.db.profile.outputFrame]:AddMessage( YCLtext, 1, 1, 1, nil, true );
		
		-- count can be 
		--  positive to advance from oldest to newest
		--  negative to advance from newest to oldest
		valid = CombatLogAdvanceEntry(-1)
		total = total + 1;
	end
	
	CombatLogQuickButtonFrameProgressBar:SetValue(CombatLogQuickButtonFrameProgressBar:GetValue() + total);
	
	if ( not valid or (CombatLogQuickButtonFrameProgressBar:GetValue() >= COMBATLOG_MESSAGE_LIMIT) ) then
		CombatLogUpdateFrame.refiltering = false
		CombatLogUpdateFrame:SetScript("OnUpdate", nil)
		CombatLogQuickButtonFrameProgressBar:Hide();
	end
end
function YurrCombatLog:CombatLog_AddEvent()
end
function YurrCombatLog:FCF_DockUpdate()
	orgFCF_DockUpdate()
	self:Blizzard_CombatLog_AdjustCombatLogHeight()
end
function YurrCombatLog:Blizzard_CombatLog_AdjustCombatLogHeight()
	local uiScale = tonumber(GetCVar("uiScale"));
	local quickButtonHeight = 0
	local yOffset = (2 + quickButtonHeight*uiScale) / uiScale;
	local xOffset = 1 + uiScale;
	local combatLogBackground = _G[format("%sBackground", COMBATLOG:GetName())];
	combatLogBackground:SetPoint("TOPLEFT", COMBATLOG, "TOPLEFT", -xOffset, yOffset);
	combatLogBackground:SetPoint("TOPRIGHT", COMBATLOG, "TOPRIGHT", xOffset, yOffset);
end
function YurrCombatLog:CombatLogQuickButtonFrame_Custom_OnShow(frame)
	frame:Hide()
end

function YurrCombatLog:setFilter()
	CombatLogResetFilter()
	
	if next(self.db.profile.watches) == nil then
		for k,v in ipairs(self.db.profile.filters) do
			if v then
				local events = ""
				local sUnit = 0
				local dUnit = 0
				if v.active then
					for k2,v2 in pairs(v) do
						if k2 ~= "name" and k2 ~= "active" and k2 ~= "sourceUnit" and k2 ~= "destUnit" and v2 then
							events = format("%s, %s", events, k2)
						end
					end
					for k2,v2 in pairs(v.sourceUnit) do
						if v2 then
							sUnit = bit.bor(sUnit, _G[k2])
						end
					end
					for k2,v2 in pairs(v.destUnit) do
						if v2 then
							dUnit = bit.bor(dUnit, _G[k2])
						end
					end
					if events == "" then sUnit = nil end
					if sUnit == 0 then sUnit = nil end
					if dUnit == 0 then dUnit = nil end
					CombatLogAddFilter(events, sUnit, dUnit)
				end
			end
		end
	else
		for k,v in pairs(self.db.profile.watches) do
			CombatLogAddFilter("ENVIRONMENTAL_DAMAGE, SWING_DAMAGE, RANGE_DAMAGE, SPELL_DAMAGE, SPELL_HEAL, SPELL_PERIODIC_HEAL, SPELL_PERIODIC_DAMAGE, DAMAGE_SHIELD, SPELL_INSTAKILL SPELL_AURA_APPLIED, SPELL_AURA_REMOVED, UNIT_DIED", nil, v)
		end
	end
	self:Blizzard_CombatLog_Refilter()
end

function YurrCombatLog:AffiliationColor(flags)
	local color
	for mask in pairs(self.db.profile.unitColors) do
		if (CombatLog_Object_IsA (flags, mask))then
			color = self.db.profile.unitColors[mask]
			break
		end
	end
	return color
end

function YurrCombatLog:ColorSpellBySchool(school, aura, suffix)
	if school == nil then
		Debug(format("%s: nil SPELLSCHOOL: ON suffix: %s", date("%X"), tostring(suffix)))
	elseif not self.db.profile.spellColors[school] then
		Debug(format("%s: MISSING SPELLSCHOOL: %s ON suffix: %s", date("%X"), school, suffix))
	end
	return spellList[suffix] or spellList[aura] or self.db.profile.spellColors[school]
end

function YurrCombatLog:DamageResult(resisted, blocked, absorbed, critical, glancing, crushing, healAbsorb)

	local resultStr
	
	if ( resisted or blocked or absorbed or critical or glancing or crushing or healAbsorb) then
		
		if ( resisted ) then
			if ( resisted < 0 ) then	--Its really a vulnerability
				resultStr = format(self.db.profile.format.VULNERABILITY, -resisted)
			else
				resultStr = format(self.db.profile.format.RESIST, resisted)
			end
		end
		if ( blocked ) then
			if ( resultStr ) then
				resultStr = format("%s%s%s", resultStr, " ", format(self.db.profile.format.BLOCK_VALUE, blocked))
				--resultStr = resultStr.." "..format(self.db.profile.format.BLOCK_VALUE, blocked)
			else
				resultStr = format(self.db.profile.format.BLOCK_VALUE, blocked)
			end
		end
		if ( absorbed or healAbsorb) then
			if ( resultStr ) then
				resultStr = format("%s%s%s", resultStr, " ", format(self.db.profile.format.ABSORB, absorbed))
				--resultStr = resultStr.." "..format(self.db.profile.format.ABSORB, absorbed)
			else
				resultStr = format(self.db.profile.format.ABSORB, absorbed)
			end
		end
		if ( glancing ) then
			if ( resultStr ) then
				resultStr = format("%s %s", resultStr, self.db.profile.format.GLANCING)
			else
				resultStr = self.db.profile.format.GLANCING;
			end
		end
		if ( crushing ) then
			if ( resultStr ) then
				resultStr = format("%s %s", resultStr, self.db.profile.format.CRUSHING)
			else
				resultStr = self.db.profile.format.CRUSHING
			end
		end
		if ( critical ) then
			local critString = self.db.profile.format.CRITICAL
			if ( spellId ) then
				critString = self.db.profile.format.CRITICAL_SPELL
			end
			if ( resultStr ) then
				resultStr = format("%s %s", resultStr, critString)
			else
				resultStr = critString
			end
		end
	end
	
	return resultStr

end

function YurrCombatLog:COMBAT_LOG_EVENT(...)
	local t = self:processEvent(...)
	_G[self.db.profile.outputFrame]:AddMessage(t)
end

function YurrCombatLog:processEvent(...)
	--Debug(...)
	
	local _, timestamp, _, srcGUID, srcUnitName, srcFlags, destGUID, destUnitName, destFlags = ...
	
	local timestampStr
	
	local event = select(3, ...)
	
	local actionString
	
	local srcUnitNameString
	
	local prefixString
	
	local spellName
	local spellNameString
	
	local suffixString
	
	local destUnitNameString
	
	local amount
	local auraStack
	local overkill
	local overhealing
	local healAbsorb
	local school
	local resisted
	local blocked
	local absorbed
	local critical
	local glancing
	local crushing
	local missType
	local amountMissed
	local powerType
	local extraSpellName
	local extraSchool
	local auraType
	local failedType
	local itemID
	local itemName
		
	local amountString
	local powerString
	local schoolStr
	local resultString
	
	local finalString
	
	if ( srcUnitName and CombatLog_Object_IsA(srcFlags, COMBATLOG_FILTER_MINE) ) then
			srcUnitName = UNIT_YOU_SOURCE;
	end
	if ( destUnitName and CombatLog_Object_IsA(destFlags, COMBATLOG_FILTER_MINE) ) then
			destUnitName = UNIT_YOU_DEST;
	end
	
	local base = 9
	local offset
	
	-- Prepare strings
	local prefix = strsub(event, 1, 5)
	local suffix = strsub(event, 6)
	
	-- Special events
	if event == "UNIT_DIED" then
		actionString = format("%s: ", TUTORIAL_TITLE25)
	elseif event == "UNIT_DESTROYED" then
		actionString = format("%s: ", ACTION_UNIT_DESTROYED)
	elseif event == "UNIT_DISSIPATES" then
		actionString = format("%s: ", L["Dissipated"])
	elseif event == "ENCHANT_APPLIED" or event == "ENCHANT_REMOVED" then
		actionString = format("%s: ", ENSCRIBE)
		offset = 0
		spellName, itemID, itemName = select(base + offset + 1, ...)
		school = 1
	elseif event == "PARTY_KILL" then
		spellName = ACTION_PARTY_KILL
		school = 1
	elseif event == "DAMAGE_SHIELD" or event == "DAMAGE_SPLIT" then
		offset = 3
		prefix = "SPELL"
		suffix = "_DAMAGE"
	elseif event == "DAMAGE_SHIELD_MISSED" then
		offset = 3
		prefix = "SPELL"
		suffix = "_MISSED"
	end
	
	if prefix == "SWING" then
		offset = 0
		spellName = MELEE
	elseif prefix == "SPELL" or prefix == "RANGE" then
		offset = 3
		spellName, school = select(11, ...)
		-- (Temporary) fix for death knight gladiator gloves generating an energize event with a zero(not nil) spellschool
		if school == 0 then school = 1 end
	elseif prefix == "ENVIR" then
		offset = 1
		srcUnitName = destUnitName
		srcFlags = destFlags
		destUnitName = nil
		destFlags = nil
		local environmentalType = select(base + offset, ...)
		spellName = L[environmentalType]
		suffix = "_DAMAGE"
	end
	if suffix == "_DAMAGE" or suffix == "_PERIODIC_DAMAGE" or suffix == "_BUILDING_DAMAGE" then
		amount, overkill, school, resisted, blocked, absorbed, critical, glancing, crushing = select(base + offset + 1, ...)
	elseif suffix == "_MISSED" or suffix == "_PERIODIC_MISSED" or suffix == "_BUILDING_MISSED" then
		missType, amountMissed = select(base + offset + 1, ...)
		school = school or 1
	elseif suffix == "_HEAL" or suffix == "_PERIODIC_HEAL" or suffix == "_BUILDING_HEAL" then
		amount, overhealing, healAbsorb, critical = select(base + offset + 1, ...)
	elseif suffix == "_ENERGIZE" or suffix == "_PERIODIC_ENERGIZE" then
		amount, powerType = select(base + offset + 1, ...)
	elseif suffix == "_DRAIN" or suffix == "_LEECH" or suffix == "_PERIODIC_DRAIN" or suffix == "_PERIODIC_LEECH" then
		amount, powerType, extraAmount = select(base + offset + 1, ...)
	elseif suffix == "_INTERRUPT" or suffix == "_DISPEL" or suffix == "_DISPEL_FAILED" or suffix == "_STOLEN" or suffix == "_AURA_BROKEN_SPELL" then
		--extraSpellID = select(base + offset + 1, ...)
		extraSpellName, extraSchool, auraType = select(base + offset + 2, ...)
	elseif suffix == "_EXTRA_ATTACKS" then
		amount = select(base + offset + 1, ...)
		spellName =  format(L.extraAttack, amount, spellName)
		school = 1
	elseif suffix == "_AURA_APPLIED" or suffix == "_AURA_REMOVED" or suffix == "_AURA_APPLIED_DOSE" or suffix == "_AURA_REMOVED_DOSE" or suffix == "_AURA_REFRESH" or suffix == "_AURA_BROKEN" then
		auraType, auraStack = select(base + offset + 1, ...)
	elseif suffix == "_CAST_FAILED" then
		failedType = select(base + offset + 1, ...)
	end
	
	if self.db.profile.blacklist[spellName] then return end
	
	-- Process strings
	if self.db.profile.timestamp then
		local stamp
		local formatting
		if self.db.profile.colorTimestamp then
			stamp = format("|cFF%s%s|r", self.db.profile.otherColors.timestampColor, date(self.db.profile.timestampFormat, timestamp)) -- SOME GARBAGE HERE
		else
			stamp = format("%s", date(self.db.profile.timestampFormat, timestamp)) -- SOME GARBAGE HERE
		end
		if self.db.profile.braceTimestamp then
			if self.db.profile.colorTimestampBrace then
				formatting = format("|cFF%s%s|r%s|cFF%s%s|r", self.db.profile.braceColors.Timestamp, self.db.profile.braces.preTimestamp, "%s", self.db.profile.braceColors.Timestamp, self.db.profile.braces.postTimestamp)
			else
				formatting = format("%s%s%s", self.db.profile.braces.preTimestamp, "%s", self.db.profile.braces.postTimestamp)
			end
			timestampStr = format(formatting, stamp)
		else
			timestampStr = stamp
		end
	else
		timestampStr = ""
	end
	
	if srcUnitName then
		local UnitStr
		local formatting
		if self.db.profile.colorSrc then
			local color
			local class
			local classColor
			if self.db.profile.colorUnitByClass then
				_, class = GetPlayerInfoByGUID(destGUID)
				classColor = classColors[class]
			else
				color = self:AffiliationColor(srcFlags)
			end
			UnitStr = format("|cFF%s%s|r", classColor or color, srcUnitName)
		else
			UnitStr = srcUnitName
		end
		if self.db.profile.braceSrcUnit then
			if self.db.profile.colorSrcBraces then
				formatting = format("|cFF%s%s|r%s|cFF%s%s|r ", self.db.profile.braceColors.Source, self.db.profile.braces.preSource, "%s", self.db.profile.braceColors.Source, self.db.profile.braces.postSource)
			else
				formatting = format("%s%s%s ", self.db.profile.braces.preSource, "%s", self.db.profile.braces.postSource)
			end
		else
			formatting = "%s "
		end
		srcUnitNameString = format(formatting, UnitStr)
	else
		srcUnitNameString = ""
	end
	
	if self.db.profile.prefix then
		local str
		str = prefixList[suffix]
		if self.db.profile.colorPrefix and str then
			local color = self:ColorSpellBySchool(school, auraType, suffix)
			prefixString = format("|cFF%s%s|r ", color, str)
		elseif str then
			prefixString = format("%s ", str)
		else
			prefixString = ""
		end
	else
		prefixString = ""
	end
	
	if spellName then
		local coloredSpell
		local formatting
		local stackedSpell
		if auraStack then stackedSpell = format("%s(%s)", spellName, auraStack) else stackedSpell = spellName end
		
		if self.db.profile.colorSpell then
			local color = self:ColorSpellBySchool(school, auraType, suffix)
			coloredSpell = format("|cFF%s%s|r", color, stackedSpell)
		else
			coloredSpell = stackedSpell
		end
		if self.db.profile.braceSpell then
			if self.db.profile.colorSpellBraces then
				formatting = format("|cFF%s%s|r%s|cFF%s%s|r", self.db.profile.braceColors.Spell, self.db.profile.braces.preSpell, "%s", self.db.profile.braceColors.Spell, self.db.profile.braces.postSpell)
			else
				formatting = format("%s%s%s", self.db.profile.braces.preSpell, "%s", self.db.profile.braces.postSpell)
			end
		else
			formatting = "%s "
		end
		spellNameString = format(formatting, coloredSpell)
	else
		spellNameString = ""
	end
	
	if self.db.profile.suffixString then
		local str
		local result
		str = suffixList[suffix] or suffixList[missType] or failedType
		
		if str then
			if self.db.profile.colorSuffix then
				local color = self:ColorSpellBySchool(school, aura, suffix)
				suffixString = format("|cFF%s%s|r ", color, str)
			else
				suffixString = format("%s ", str)
			end
		else
			suffixString = ""
		end
	else
		suffixString = ""
	end
	
	if destUnitName then
		local UnitStr
		local formatting
		if self.db.profile.colorDest then
			local color
			local class
			local classColor
			if self.db.profile.colorUnitByClass then
				_, class = GetPlayerInfoByGUID(destGUID)
				classColor = classColors[class]
			else
				color = self:AffiliationColor(destFlags)
			end
			UnitStr = format("|cFF%s%s|r", classColor or color, destUnitName)
		else
			UnitStr = destUnitName
		end
		if self.db.profile.braceDestUnit then
			if self.db.profile.colorDestBraces then
				formatting = format("|cFF%s%s|r%s|cFF%s%s|r ", self.db.profile.braceColors.Dest, self.db.profile.braces.preDest, "%s", self.db.profile.braceColors.Dest, self.db.profile.braces.postDest)
			else
				formatting = format("%s%s%s ", self.db.profile.braces.preDest, "%s", self.db.profile.braces.postDest)
			end
			destUnitNameString = format(formatting, UnitStr)
		else
			destUnitNameString = format("%s ", UnitStr)
		end
	else
		destUnitNameString = ""
	end
	
	if amount then
		local amountResult
		local AmountStr
		local braceFormatting
		local critFormatting
		local dotFormatting
		
		if self.db.profile.overhealing and type(overhealing) == "number" and overhealing > 0 then
			amountResult = format("%d(+%d)", amount - overhealing, overhealing) -- SOME GARBAGE HERE
		elseif self.db.profile.overkill and type(overkill) == "number" and overkill > 0 then
			amountResult = format("%d(+%d)", amount - overkill, overkill) -- SOME GARBAGE HERE
		else
			amountResult = amount
		end
		
		if suffix == "_ENERGIZE" or suffix == "_PERIODIC_ENERGIZE" then
			amountResult = format("+ %s", amountResult)
		end
		
		if self.db.profile.colorAmount and school then
			local color = self:ColorSpellBySchool(school, auraType, suffix)
			AmountStr = format("|cFF%s%s|r", color, amountResult)
		else
			AmountStr = amountResult
		end
		
		if self.db.profile.braceAmount then
			if self.db.profile.colorAmountBraces then
				braceFormatting = format("|cFF%s%s|r%s|cFF%s%s|r ", self.db.profile.braceColors.Amount, self.db.profile.braces.preAmount, "%s", self.db.profile.braceColors.Amount, self.db.profile.braces.postAmount)
			else
				braceFormatting = format("%s%s%s ", self.db.profile.braces.preAmount, "%s", self.db.profile.braces.postAmount)
			end
		else
			braceFormatting = "%s "
		end
		
		if self.db.profile.markCrits and critical then
			critFormatting = format("%s%s%s", self.db.profile.braces.preCrit, "%s", self.db.profile.braces.postCrit)
		else
			critFormatting = "%s"
		end
		
		if self.db.profile.braceAmountDots and (suffix == "_PERIODIC_DAMAGE" or suffix == "_PERIODIC_HEAL") then
			if self.db.profile.colorAmountDotBraces then
				dotFormatting = format("|cFF%s%s|r%s|cFF%s%s|r", self.db.profile.braceColors.AmountDots, self.db.profile.braces.preAmountDots, "%s", self.db.profile.braceColors.AmountDots, self.db.profile.braces.postAmountDots)
			else
				dotFormatting = format("%s%s%s", self.db.profile.braces.preAmountDots, "%s", self.db.profile.braces.postAmountDots)
			end
		else
			dotFormatting = "%s"
		end
		
		amountNameString = format(braceFormatting, format(critFormatting, format(dotFormatting, AmountStr)))
	else
		amountNameString = ""
	end
	
	if extraSpellName then
		local color = self:ColorSpellBySchool(extraSchool, auraType)
		amountNameString = format("|cFF%s%s|r ", color, extraSpellName)
	end
	if itemName then
		amountNameString = itemName
	end
	
	if self.db.profile.powerType and powerType then
		powerString = format("%s ", powerTypes[powerType])
	else
		powerString = ""
	end
	
	if self.db.profile.schoolText and school then
		local coloredSchool
		local schoolText
		if self.db.profile.colorSchoolText then
			local color = self:ColorSpellBySchool(school)
			coloredSchool = format("|cFF%s%s|r", color, schools[school])
		else
			coloredSchool = schools[school]
		end
		if self.db.profile.braceSchoolText then
			if self.db.profile.colorSchoolTextBraces then
				local color = self:ColorSpellBySchool(school)
				schoolText = format("|cFF%s%s|r%s|cFF%s%s|r ", color, self.db.profile.braces.preSchoolText, coloredSchool, color, self.db.profile.braces.postSchoolText)
			else
				schoolText = format("%s%s%s ", self.db.profile.braces.preSchoolText, coloredSchool, self.db.profile.braces.postSchoolText)
			end
		else
			schoolText = coloredSchool
		end
		schoolStr = schoolText
	else
		schoolStr = ""
	end
	
	if self.db.profile.resultString then
		if missType == "RESIST" then
			resisted = amountMissed
		elseif missType == "BLOCK" then
			blocked = amountMissed
		elseif missType == "ABSORB" then
			absorbed = amountMissed
		elseif missType ~= "MISS" then
			resultString = self.db.profile.format[missType]
		end
			
		if suffix == "_DISPEL_FAILED" then
			resultString = L.failed
		end
		
		if not resultString then resultString = self:DamageResult(resisted, blocked, absorbed, critical, glancing, crushing, healAbsorb) or "" end
	else
		resultString = ""
	end

	finalString = format("%s%s%s%s%s%s%s%s%s%s%s", timestampStr, actionString or "", srcUnitNameString, prefixString, spellNameString, suffixString, destUnitNameString, amountNameString, powerString, schoolStr, resultString)
	return finalString
end

